use crate::config::Config;
use crate::zabbix::api::get_zabbix_problems;
use crate::zabbix::problems::ZabbixLayout;
use launchy::Color;
use launchy::{self, Canvas, CanvasLayout, CanvasLayoutPoller, MsgPollingWrapper, Pad};

pub fn initpad() -> (CanvasLayout<'static>, CanvasLayoutPoller) {
    let (mut canvas, poller) = launchy::CanvasLayout::new_polling();
    match canvas.add_by_guess_rotated::<launchy::mini::Canvas>(0, 0, launchy::Rotation::None) {
        Ok(o) => o,
        _ => {
            eprintln!("Failed to connect to device, check USB connection");
            std::process::exit(1);
        }
    };
    println!("Connected to device using USB");
    clear(&mut canvas);
    (canvas, poller)
}

pub fn _input(canvas: &mut CanvasLayout, poller: &mut CanvasLayoutPoller) {
    for color in (0u64..).map(|f| Color::red_green_color(f as f32 / 60.0 / 2.5)) {
        for msg in poller.iter_for_millis(17).filter(|msg| msg.is_press()) {
            canvas[msg.pad()] = color;
            println!("{:?}", msg.pad())
        }
        let _res = canvas.flush();
    }
}

pub fn draw(canvas: &mut CanvasLayout, datamatrix: &mut ZabbixLayout, cfg: &mut Config) {
    println!("Refresh rate is {} seconds", cfg.refresh.unwrap());
    loop {
        let zabbix_data = get_zabbix_problems(cfg);
        datamatrix.compute(&zabbix_data);
        let mut y = 1i32;
        for (x, j) in datamatrix.layout.iter().enumerate() {
            if y % 8 == 0 {
                y += 1;
            };
            let p = Pad { x: x as i32, y: y };
            let c = match j.status {
                4 => Color::RED,
                3 => Color::YELLOW,
                2 => Color::WHITE,
                _ => Color::GREEN,
            };
            canvas[p] = c;
        }
        let _a = canvas.flush();
        std::thread::sleep(std::time::Duration::from_secs(cfg.refresh.unwrap_or(5)));
    }
}

fn _effect(canvas: &mut CanvasLayout, poller: &mut CanvasLayoutPoller) {
    for color in (0u64..).map(|f| Color::red_green_color(f as f32 / 60.0 / 2.5)) {
        for msg in poller.iter_for_millis(17).filter(|msg| msg.is_press()) {
            canvas[msg.pad()] = color;
            println!("{:?}", msg.pad())
        }
        let _res = canvas.flush();

        canvas.iter().for_each(|pad| {
            let surrounding_color = pad
                .neighbors_5()
                .iter()
                .map(|&p| canvas.get(p).unwrap_or(Color::GREEN))
                .sum::<Color>()
                / 5.0
                / 1.05;

            canvas[pad] = canvas[pad].mix(surrounding_color, 0.4);
        });
    }
}

fn clear(canvas: &mut CanvasLayout) {
    for a in 0..8 {
        for b in 0..8 {
            canvas[Pad { x: a, y: b }] = Color {
                r: 0f32,
                g: 0f32,
                b: 1f32,
            };
        }
    }
    let _res = canvas.flush();
}
