use crate::config::Config;
use serde_json::{json, Value};

pub const ZABBIX_API_VERSION: &'static str = "2.0";
pub const ZABBIX_API_ID: i32 = 1;

pub fn get_zabbix_authtoken(cfg: &mut Config) {
    let body = query_auth_token(&cfg.username, &cfg.password);
    let resp = reqwest::blocking::Client::new()
        .post(&cfg.server)
        .json(&body)
        .send()
        .unwrap();

    let values: Value = resp.json().unwrap();
    cfg.authtoken = Some(values["result"].as_str().unwrap().to_string());
}

pub fn get_zabbix_problems(cfg: &mut Config) -> Value {
    let body = query_problems(
        &cfg.authtoken.as_ref().unwrap_or(&String::from("")),
        cfg.limit.unwrap(),
    );
    let resp = reqwest::blocking::Client::new()
        .post(&cfg.server)
        .json(&body)
        .send()
        .unwrap();

    resp.json().unwrap()
}

pub fn query_auth_token(zabbix_username: &String, zabbix_password: &String) -> Value {
    let zabbix_api_function = "user.login";
    json!({
        "jsonrpc": ZABBIX_API_VERSION,
        "method": zabbix_api_function,
        "params": {
            "user": zabbix_username,
            "password": zabbix_password
        },
        "id": ZABBIX_API_ID
    })
}

pub fn query_problems(zabbix_token: &String, zabbix_limit: i64) -> Value {
    let zabbix_api_function = "problem.get";
    json!({
        "jsonrpc": ZABBIX_API_VERSION,
        "method": zabbix_api_function,
        "params": {
            "output": "extend",
            "selectAcknowledges": "extend",
            "selectTags": "extend",
            "selectSuppressionData": "extend",
            "recent": "true",
            "sortfield": ["eventid"],
            "sortorder": "DESC",
            "limit": zabbix_limit
        },
        "auth": zabbix_token,
        "id": ZABBIX_API_ID
    })
}
